@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 1996-2015 The Squid Software Foundation and contributors
+ * Copyright (C) 1996-2016 The Squid Software Foundation and contributors
  *
  * Squid software is distributed under GPLv2+ license and includes
  * contributions from numerous individuals and organizations.
@@ -24,6 +24,15 @@
 #define OLD_FREE free
 #undef free
 #endif
+
+#if __clang__
+// workaround for clang complaining of unknown attributes in libxml2 on fedora22
+#ifdef LIBXML_ATTR_ALLOC_SIZE
+#undef LIBXML_ATTR_ALLOC_SIZE
+#endif
+#define LIBXML_ATTR_ALLOC_SIZE(x)
+#endif /* __clang__ */
+
 #if HAVE_LIBXML_PARSER_H
 #include <libxml/parser.h>
 #endif
